extends Node

export (PackedScene) var mob_scene

var camera_move_left = false
var camera_move_right = false
var camera_move_up = false
var camera_move_down = false

var mouse_pos = Vector2()
var mouse_old_pos = Vector2(0, 0)
var rotate_camera = false

# Called when the node enters the scene tree for the first time.
func _ready():
	randomize()
	$UserInterface/Retry.hide()


# Called every frame. 'delta' is the elapsed time since the previous frame.
func _process(delta):
	var _speed = Vector3(0.0, 0.0, 0.0)
	
	if camera_move_left:
		_speed.x = -10.0
	
	if camera_move_right:
		_speed.x = 10.0
	
	if camera_move_up:
		_speed.z = -10.0
		_speed.y = -10.0
	
	if camera_move_down:
		_speed.z = 10.0
		_speed.y = 10.0
	
	$CameraPivot/Camera.global_translate(_speed * delta)
	
	if rotate_camera:
		mouse_old_pos = mouse_pos
		mouse_pos = get_viewport().get_mouse_position()
		
		var _yaw = sign(mouse_pos.x - mouse_old_pos.x) * 100.0
		var _pitch = sign(mouse_pos.y - mouse_old_pos.y) * 100.0
		
		#print(mouse_pos.x, " ", mouse_old_pos.x, " ", _yaw)
		
		$CameraPivot/Camera.rotate_y(deg2rad(-_yaw * delta))
		$CameraPivot/Camera.rotate_object_local(Vector3(1,0,0), deg2rad(-_pitch * delta))


func _input(event):
	if event is InputEventMouseButton:
		if event.button_index == BUTTON_LEFT and event.pressed:
			rotate_camera = true
		if event.button_index == BUTTON_LEFT and !event.pressed:
			rotate_camera = false
	elif event is InputEventMouseMotion:
		pass

func _unhandled_input(event):
	if event.is_action_pressed("restart_game") and $UserInterface/Retry.visible:
		# warning-ignore:return_value_discarded
		get_tree().reload_current_scene()
	
	if event.is_action_pressed("move_camera_left"):
		camera_move_left = true
	
	if event.is_action_released("move_camera_left"):
		camera_move_left = false
	
	if event.is_action_pressed("move_camera_right"):
		camera_move_right = true
	
	if event.is_action_released("move_camera_right"):
		camera_move_right = false
	
	if event.is_action_pressed("move_camera_up"):
		camera_move_up = true
	
	if event.is_action_released("move_camera_up"):
		camera_move_up = false
	
	if event.is_action_pressed("move_camera_down"):
		camera_move_down = true
	
	if event.is_action_released("move_camera_down"):
		camera_move_down = false
	
	#if event.is_action_pressed("rotate_camera"):
	#	rotate_camera = true
	
	#if event.is_action_released("rotate_camera"):
	#	rotate_camera = false


func _on_MobTimer_timeout():
	var mob = mob_scene.instance()
	var mob_spawn_location = get_node("SpawnPath/SpawnLocation")	
	mob_spawn_location.unit_offset = randf()
	
	var player_position = $Player.transform.origin
	mob.initialize(mob_spawn_location.translation, player_position)
		
	add_child(mob)
	mob.connect("squashed", $UserInterface/ScoreLabel, "_on_Mob_squashed")


func _on_Player_hit():
	if $Player.player_enemy_hits < 1:
		$MobTimer.stop()
		$UserInterface/Retry.show()
